# Examples 9.6 - 9.8 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################


##### Example 9.6
# Example from Maltamo et al 2012
library(lmfor)
data(alsTree)
dat<-rbind(
     cbind(plot=alsTree$plot,tree=alsTree$tree,index=1,y=alsTree$DBH,
           conD=1,hmaxD=alsTree$hmax,a_hmeanD=alsTree$a_hmean,
           sqh30D=sqrt(alsTree$h30),
           conH=0,hmaxH=0,h80H=0,
           conV=0,hmaxV=0,lna_h30V=0,lnh70V=0,
           conHDB=0,hmaxHDB=0,a_h70HDB=0,
           conHCB=0,h20HCB=0,a_hmeanHCB=0,lna_vegHCB=0),
     cbind(plot=alsTree$plot,tree=alsTree$tree,index=2,y=alsTree$H,
           0,0,0,0,
           conH=1,hmaxH=alsTree$hmax,h80H=alsTree$h80,
           0,0,0,0,0,0,0,0,0,0,0),
     cbind(plot=alsTree$plot,tree=alsTree$tree,index=3,y=log(alsTree$V),
           0,0,0,0,0,0,0,
           conV=1,hmaxV=alsTree$hmax,lna_hp30V=log(alsTree$a_h30),
           lnh70V=log(alsTree$h70),
           0,0,0,0,0,0,0),
     cbind(alsTree$plot,alsTree$tree,index=4,y=sqrt(alsTree$HDB),
           0,0,0,0,0,0,0,0,0,0,0,
           conHDB=1,hmaxHDB=alsTree$hmax,a_h70HDB=alsTree$a_h70,
           0,0,0,0),
     cbind(alsTree$plot,alsTree$tree,index=5,y=alsTree$HCB,
           0,0,0,0,0,0,0,0,0,0,0,0,0,0,
           conHCB=1,h20HCB=alsTree$h20,a_hmeanHCB=alsTree$a_hmean,
           lna_vegHCB=log(alsTree$a_veg)))

dat<-as.data.frame(dat)
dat<-dat[!is.na(dat$y),]
head(dat)

# Fit a system of mixed-effects models
# All component models have random constant and slope at plot level.
# Cross-model covariances of random effects are estimated, too
# All components have their own constant residual variances
# All cross-model covariances of residual errors are also estimated.
# Be patient, this will take time! 
# When you run this for the second time, change TRUE to FALSE below to read the fitted object from computer memory.
# The textbooks seems to have this model both with name syssur2 and syssur. 
if (FALSE) {
syssur <- lme(y ~ -1 + conD + hmaxD + a_hmeanD + sqh30D + conH + hmaxH + h80H + 
				conV + hmaxV + lna_h30V + lnh70V + conHDB + hmaxHDB + a_h70HDB + 
				conHCB + h20HCB + a_hmeanHCB + lna_vegHCB,			
		data = dat,
		random = ~ -1 + conD + hmaxD  + conH + hmaxH + conV + hmaxV + 
				conHDB + hmaxHDB + conHCB + h20HCB |plot,
		weights=varIdent(form = ~1|index),
		corr=corSymm(form = ~index|plot/tree),
		control=list(msVerbose=TRUE,
				maxIter=5000,msMaxIter=5000,
				msMaxEval=5000,niterEM=50))
save(syssur,file="syssur.RData")
} else {
	load("syssur.RData")
}
syssur

#####
##### Example 9.7
# Searle and Rounsaville
modD<-lme(DBH~hmax,random=~1|plot,data=alsTree)
modH<-lme(H~hmax,random=~1|plot,data=alsTree)
modDplusH<-lme(I(DBH+H)~hmax,random=~1|plot,data=alsTree)

fixef(modDplusH)
fixef(modD)+fixef(modH)

covb<-(getVarCov(modDplusH)[1]-getVarCov(modD)[1]-getVarCov(modH)[1])/2
corb<-covb/sqrt(getVarCov(modD)[1]*getVarCov(modH)[1])
cove<-(modDplusH$sigma^2-modD$sigma^2-modH$sigma^2)/2
core<-cove/(modD$sigma*modH$sigma)
corb
core

datDH<-rbind(
		cbind(plot=alsTree$plot,tree=alsTree$tree,index=1,y=alsTree$DBH,
				conD=1,hmaxD=alsTree$hmax,a_hmeanD=alsTree$a_hmean,
				sqh30D=sqrt(alsTree$h30),conH=0,hmaxH=0,h80H=0),
		cbind(plot=alsTree$plot,tree=alsTree$tree,index=2,y=alsTree$H,0,0,0,0,
				conH=1,hmaxH=alsTree$hmax,h80H=alsTree$h80))
datDH<-as.data.frame(datDH)

sysDH <- lme(y ~ -1 + conD + hmaxD + conH + hmaxH,			
		data = datDH,
		random = ~ -1 + conD + conH|plot,
		weights=varIdent(form = ~1|index),
		corr=corSymm(form = ~1|plot/tree))
sysDH
#####


##### Example 9.8: Prediction
obs<-alsTree[alsTree$plot==1,][c(2,5,7),c(1:3,6:16)]
options(width=100,digits=4)
obs

D<-getVarCov(syssur)
corepsilon<-corMatrix(syssur$modelStruct[2]$corStruct)[1][[1]]
sdepsilon<-syssur$sigma*c(1,exp(coef(syssur$modelStruct[3]$varStruct)))
Sigma<-diag(sdepsilon)%*%corepsilon%*%diag(sdepsilon)

# Assume that DBH, HCB, and HDB are known for these three trees
# Predict plot effects for al models.
library(magic)
ntrees<-3
Z1234<-cbind(1,obs[,"hmax"])
Z5<-cbind(1,obs[,"h20"]) 
Zo<-adiag(Z1234,Z1234,Z5)
C<-D[,-c(3,4,5,6)]
Do<-D[-c(3,4,5,6),-c(3,4,5,6)]
Ro<-Sigma[-c(2,3),-c(2,3)]%x%diag(ntrees)

Xo<-adiag(cbind(1,obs$hmax,obs$a_hmean,sqrt(obs$h30)), 
          cbind(1,obs$hmax,obs$a_h70),
          cbind(1,obs$h20,obs$a_hmean,log(obs$a_veg))) 

beta<-fixef(syssur)
betao<-beta[-seq(5,11)]
muo<-Xo%*%betao
yo<-c(obs$DBH,sqrt(obs$HDB),obs$HCB)

round(D,3)
Ro
round(C,3)
Zo
Xo
yo
t(muo)
		
b<-C%*%t(Zo)%*%solve(Zo%*%Do%*%t(Zo)+Ro)%*%(yo-muo)
varbb<-D-C%*%t(Zo)%*%solve(Zo%*%t(Do)%*%t(Zo)+Ro)%*%Zo%*%t(C)
t(b)
round(varbb,3)

new<-alsTree[alsTree$plot==1,][10:14,]
options(digits=3)
new

X<-adiag(cbind(1,new$hmax,new$a_hmean,sqrt(new$h30)), 
         cbind(1,new$hmax,new$h80),
         cbind(1,new$hmax,log(new$a_h30),log(new$h70)),
         cbind(1,new$hmax,new$a_h70),
         cbind(1,new$h20,new$a_hmean,log(new$a_veg))) 
Z1234<-cbind(1,new[,"hmax"])
Z5<-cbind(1,new[,"h20"]) 
Z<-adiag(Z1234,Z1234,Z1234,Z1234,Z5)

predf<-X%*%beta
predfr<-predf+Z%*%b

res1<-cbind(predf,predfr)
res1[11:15,]<-exp(res1[11:15,])
res1[16:20,]<-exp(res1[16:20,])

res<-cbind(matrix(res1,nrow=5),new[,3:7])
colnames(res)<-rep(c("DBH","H","V","HDB","HCB"),3)
res

B<-Sigma[,-c(2,3)]%x%diag(ntrees)

V1<-adiag(R,D)
V12<-rbind(B,C%*%t(Zo))
V2<-Zo%*%Do%*%t(Zo)+Ro
h1hat<-V12%*%solve(V2)%*%(yo-muo)

# The predicted residuals for DBH, H, V, HDB, and HCB
matrix(h1hat[1:15],ncol=5)
# For observed responses, the residuals are y-(xbeta+zb)
t(yo-muo-Zo%*%b[-c(3:6)])
#####
